//+------------------------------------------------------------------+
//|                                                        DTOSC.mq5 |
//|                        Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_plots   2
//--- plot SK
#property indicator_label1  "SK"
#property indicator_type1   DRAW_LINE
#property indicator_color1  Red
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot SD
#property indicator_label2  "SD"
#property indicator_type2   DRAW_LINE
#property indicator_color2  Blue
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

#include <MovingAverages.mqh>
double         MAonArrayBuffer[];
//--- input parameters
input int      PeriodRSI=13;
input int      PeriodStoch=8;
input int      PeriodSK=5;
input int      PeriodSD=3;
         //    0 = SMA
         //    1 - EMA
         //    2 - SMMA
         //    3 - LWMA
input int      MAMode=0;
input int      OSLevel=75;
input int      OBLevel=25;

//--- indicator buffers
double         SKBuffer[];
double         SDBuffer[];

double         StoRSIBuffer[];
double         RSIBuffer[];

int            rsihandle;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
  
   rsihandle=iRSI(NULL ,0,PeriodRSI,PRICE_CLOSE);
//--- indicator buffers mapping
   SetIndexBuffer(0,SKBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,SDBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,StoRSIBuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,RSIBuffer,INDICATOR_CALCULATIONS);
   //placing levels
   IndicatorSetInteger(INDICATOR_DIGITS, 2);
   IndicatorSetInteger(INDICATOR_LEVELS, 2);
   IndicatorSetDouble(INDICATOR_LEVELVALUE,0,OBLevel);
   IndicatorSetDouble(INDICATOR_LEVELVALUE,1,OSLevel);
//--- set maximum and minimum for subwindow 
   IndicatorSetDouble(INDICATOR_MINIMUM,0);
   IndicatorSetDouble(INDICATOR_MAXIMUM,100);
//--- name for DataWindow and indicator subwindow label
   IndicatorSetString(INDICATOR_SHORTNAME,"DTOSC ("+(string)PeriodRSI+","+(string)PeriodStoch+","+(string)PeriodSK+","+(string)PeriodSD+")");
   PlotIndexSetString(0,PLOT_LABEL,"OB");
   PlotIndexSetString(1,PLOT_LABEL,"OS");
//--- sets first bar from what index will be drawn
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,PeriodRSI - 2);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,PeriodRSI - 2);
   
    ArraySetAsSeries(MAonArrayBuffer,true);
    ArraySetAsSeries(SKBuffer,true);
    ArraySetAsSeries(SDBuffer,true);
    ArraySetAsSeries(StoRSIBuffer,true);
    ArraySetAsSeries(RSIBuffer,true);
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
  {
     
//---
   int i,limit;
   int counted_bars =  prev_calculated;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
             limit=Bars(Symbol(), PERIOD_CURRENT) - counted_bars-1;
             
   CopyBuffer(rsihandle,0,0,Bars(Symbol(), PERIOD_CURRENT),RSIBuffer);
   ArrayResize(SKBuffer,Bars(Symbol(), PERIOD_CURRENT));
   ArrayResize(SDBuffer,Bars(Symbol(), PERIOD_CURRENT));
   ArrayResize(StoRSIBuffer,Bars(Symbol(), PERIOD_CURRENT));
   
   for(i=limit; i>=0; i--)
   {
      double LLV = RSIBuffer[ArrayMinimum(RSIBuffer,i,PeriodStoch)];
      double HHV = RSIBuffer[ArrayMaximum(RSIBuffer,i,PeriodStoch)];
      if ((HHV-LLV)!=0)
            StoRSIBuffer[i] = 100.0*((RSIBuffer[i] - LLV)/(HHV - LLV));
      else  StoRSIBuffer[i] = 0;
   }  
   //ExponentialMAOnBuffer(Bars,0,0,MAPeriod,OriginalBuffer,MAonArrayBuffer);
    
   for(i=limit; i>=0; i--) SKBuffer[i]=iMAOnArray(StoRSIBuffer,0,PeriodSK,0,MAMode,i);
   //for(i=limit; i>=0; i--) SKBuffer[i] = ExponentialMAOnBuffer(Bars,0 ,0 ,PeriodSK ,StoRSIBuffer, MAonArrayBuffer);
   for(i=limit; i>=0; i--) SDBuffer[i]=iMAOnArray(    SKBuffer,0,PeriodSD,0,MAMode,i);
  // for(i=limit; i>=0; i--) SDBuffer[i] = ExponentialMAOnBuffer(Bars,0 ,0 ,PeriodSD ,StoRSIBuffer, MAonArrayBuffer);

//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+

double iMAOnArray(double &array[],
                      int total,
                      int period,
                      int ma_shift,
                      int ma_method,
                      int shift)
  {
   double buf[],arr[];
   if(total==0) total=ArraySize(array);
   if(total>0 && total<=period) return(0);
   if(shift>total-period-ma_shift) return(0);
   switch(ma_method)
     {
      case MODE_SMA :
        {
         total=ArrayCopy(arr,array,0,shift+ma_shift,period);
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0;
         int    i,pos=total-1;
         for(i=1;i<period;i++,pos--)
            sum+=arr[pos];
         while(pos>=0)
           {
            sum+=arr[pos];
            buf[pos]=sum/period;
            sum-=arr[pos+period-1];
            pos--;
           }
         return(buf[0]);
        }
      case MODE_EMA :
        {
         if(ArrayResize(buf,total)<0) return(0);
         double pr=2.0/(period+1);
         int    pos=total-2;
         while(pos>=0)
           {
            if(pos==total-2) buf[pos+1]=array[pos+1];
            buf[pos]=array[pos]*pr+buf[pos+1]*(1-pr);
            pos--;
           }
         return(buf[shift+ma_shift]);
        }
      case MODE_SMMA :
        {
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0;
         int    i,k,pos;
         pos=total-period;
         while(pos>=0)
           {
            if(pos==total-period)
              {
               for(i=0,k=pos;i<period;i++,k++)
                 {
                  sum+=array[k];
                  buf[k]=0;
                 }
              }
            else sum=buf[pos+1]*(period-1)+array[pos];
            buf[pos]=sum/period;
            pos--;
           }
         return(buf[shift+ma_shift]);
        }
      case MODE_LWMA :
        {
         if(ArrayResize(buf,total)<0) return(0);
         double sum=0.0,lsum=0.0;
         double price;
         int    i,weight=0,pos=total-1;
         for(i=1;i<=period;i++,pos--)
           {
            price=array[pos];
            sum+=price*i;
            lsum+=price;
            weight+=i;
           }
         pos++;
         i=pos+period;
         while(pos>=0)
           {
            buf[pos]=sum/weight;
            if(pos==0) break;
            pos--;
            i--;
            price=array[pos];
            sum=sum-lsum+price*period;
            lsum-=array[i];
            lsum+=price;
           }
         return(buf[shift+ma_shift]);
        }
      default: return(0);
     }
   return(0);
  }